searchNodes=[{"ref":"PlugSignature.html","title":"PlugSignature","type":"module","doc":"Server side implementation of IETF HTTP signature draft (https://tools.ietf.org/html/draft-cavage-http-signatures-11), as a reusable Plug. Supports the following algorithms: &quot;hs2019&quot;, using ECDSA, RSASSA-PSS or HMAC (all with SHA-512) &quot;rsa-sha256&quot;, using RSASSA-PKCS1-v1_5 &quot;ecdsa-sha256&quot; &quot;hmac-sha256&quot; &quot;rsa-sha1&quot;, using RSASSA-PKCS1-v1_5 ECDSA signatures with hs2019/ecdsa-sha256 are accepted in ASN.1 or raw r/s format, for maximum interoperability. Signature validity time window Requests signed according to the &quot;hs2019&quot; algorithm should include a &#39;created&#39; and/or &#39;expires&#39; timestamp. When present in the Authorization header, the &#39;created&#39; parameter is checked against the configured validity (unless it is set to :infinity) and the &#39;expires&#39; parameter is checked against the current time. At least one of these parameters should be included in the signature calculation, through the respecive pseudo-header. The validity check based on &#39;created&#39; and/or &#39;expires&#39; takes place regardless of whether the parameters were signed. For legacy algorithms, signature validity should be checked using the HTTP &#39;Date&#39; header. This happens for all algorithms (including &quot;hs2019&quot;) if and only if when the header value is included in the signature. Signing the request body The HTTP &#39;Digest&#39; header can be used to protect the integrity of the request body and include it in the signature. The PlugSignature Plug treats the &#39;Digest&#39; header as any other header, i.e. it verifies the integrity of the header value if included in the signature, but it does not verify the integrity of the request body itself. Use a Plug such as PlugBodyDigest to handle the processing of the &#39;Digest&#39; header. Options :callback_module (mandatory) - the name of a callback module implementing the PlugSignature.Callback behaviour; this module must implement the PlugSignature.Callback.client_lookup/3 callback :algorithms - the signature algorithms, as defined in the IETF specification; a list containing one or more of: &quot;hs2019&quot; (default) Legacy algorithms: &quot;rsa-sha256&quot; &quot;rsa-sha1&quot; &quot;ecdsa-sha256&quot; &quot;hmac-sha256&quot; The first algorithm in the list is considered the default algorithm: if a client does not specify an algorithm the request is assumed to be signed using this algorithm :headers - the minimum set of (pseudo-)headers that need to be signed; defaults to the request timestamp, taken from the &#39;created&#39; signature parameter or the HTTP &#39;Date&#39; header, depending on the selected algorithm :validity - a Range defining the timeframe (in seconds) after signing during which the signature is considered valid; set to :infinity to disable, relying solely on the :expires parameter; defaults to -300..30, meaning a signature can be up to 5 minutes old, or up to 30 seconds in the future :legacy - a keyword list, used to override the :headers and/or :validity options for legacy algorithms (those other than &quot;hs2019&quot;); this may be necessary when these options are set to values not supported by the legacy algorithms; see the examples below :on_success - an optional callback for updating the Plug.Conn state upon success; possible values include: nil (the default) - do nothing {PlugSignature, :assign_client, [key]} - assign the client, as returned by the PlugSignature.Callback.client_lookup/3 callback, in the Plug.Conn struct to the specified key {m, f, a} - call the function identified by the atom f in module m; the function receives the current Plug.Conn struct and the client returned by the c:PlugSignature.Callbacks.client_lookup/3 callback, along with any additional parameters in the list a, and is expected to return the updated Plug.Conn struct :on_failure - an optional callback for updating the Plug.Conn state upon failure; possible values include: {PlugSignature, :failure, []} (the default) - halt the connection with an appropriate response; see failure/3 below {m, f, a} - call the function identified by the atom f in module m; the function receives the current Plug.Conn struct, the error reason (see t:error_reason/0),the selected algorithm (a string) and a list of required headers (strings, for possible use in a &#39;WWW-Authenticate&#39; response header), along with any additional parameters in the list a; it is expected to return the updated Plug.Conn struct; see the implementation of failure/4 for an example nil - do nothing Examples # Minimal example relying on defaults: &quot;hs2019&quot; algorithm only, with # minimal &quot;(created)&quot; headers and default validity: plug PlugSignature, callback_module: MyApp.SignatureAuth # More realistic example with custom header configuration; &quot;hs2019&quot; # only: plug PlugSignature, callback_module: MyApp.SignatureAuth, headers: &quot;(request-target) (created) host digest&quot; # Using legacy algorithms only, with custom header set: plug PlugSignature, callback_module: MyApp.SignatureAuth, algorithms: [&quot;ecdsa-sha256&quot;, &quot;rsa-sha256&quot;], headers: &quot;(request-target) date host&quot; # Mix of &quot;hs2019&quot; and legacy algorithms, using &#39;expires&#39; rather than # &#39;created&#39; to verify validity for &quot;hs2019&quot;; the `:legacy` option is # necessary since the `:headers` and `:validity` values are not valid # for legacy algoritms: plug PlugSignature, callback_module: MyApp.SignatureAuth, headers: &quot;(request-target) (expires) host digest&quot;, validity: :infinity, legacy: [ headers: &quot;(request-target) date host digest&quot;, validity: -300..30, ]"},{"ref":"PlugSignature.html#assign_client/3","title":"PlugSignature.assign_client/3","type":"function","doc":"Success function that assigns the authenticated client under the specified key in the Plug.Conn struct."},{"ref":"PlugSignature.html#failure/4","title":"PlugSignature.failure/4","type":"function","doc":"The default failure function. It logs the failure reason, returns a 401 &#39;Unauthorized&#39; response with a &#39;WWW-Authenticate&#39; response header listing the supported algorithms, and halts the connection."},{"ref":"PlugSignature.Callback.html","title":"PlugSignature.Callback","type":"behaviour","doc":"Behaviour for the callback module that implements client and credential lookup for PlugSignature. Example defmodule MyApp.SignatureAuth do @behaviour PlugSignature.Callback @impl true def client_lookup(key_id, &quot;hs2019&quot;, _conn) do # ... {:ok, client, client.hmac_secret} end end"},{"ref":"PlugSignature.Callback.html#c:client_lookup/3","title":"PlugSignature.Callback.client_lookup/3","type":"callback","doc":"Takes the keyId from the parsed Authorization header, the algorithm name and the Plug.Conn struct, and returns a success or error tuple. In case of success, an application-specific term is returned that identifies the client, along with that client&#39;s credentials (a public key or HMAC secret). The Plug.Conn struct may be used to select the relevant client, but it cannot be modified. For instance, the hostname of the request may be needed to select the correct client in a multi-tennant application."},{"ref":"PlugSignature.ConfigError.html","title":"PlugSignature.ConfigError","type":"exception","doc":"Exception raised in case of an invalid configuration."},{"ref":"PlugSignature.ConnTest.html","title":"PlugSignature.ConnTest","type":"module","doc":"Helpers for testing HTTP signatures with Plug/Phoenix."},{"ref":"PlugSignature.ConnTest.html#signed_connect/4","title":"PlugSignature.ConnTest.signed_connect/4","type":"macro","doc":"Makes a signed request to a Phoenix endpoint. The last argument is a keyword list with signature options, with the :key and :key_id options being mandatory. For other options, please see with_signature/4. Requires Phoenix.ConnTest."},{"ref":"PlugSignature.ConnTest.html#signed_delete/4","title":"PlugSignature.ConnTest.signed_delete/4","type":"macro","doc":"Makes a signed request to a Phoenix endpoint. The last argument is a keyword list with signature options, with the :key and :key_id options being mandatory. For other options, please see with_signature/4. Requires Phoenix.ConnTest."},{"ref":"PlugSignature.ConnTest.html#signed_get/4","title":"PlugSignature.ConnTest.signed_get/4","type":"macro","doc":"Makes a signed request to a Phoenix endpoint. The last argument is a keyword list with signature options, with the :key and :key_id options being mandatory. For other options, please see with_signature/4. Requires Phoenix.ConnTest."},{"ref":"PlugSignature.ConnTest.html#signed_head/4","title":"PlugSignature.ConnTest.signed_head/4","type":"macro","doc":"Makes a signed request to a Phoenix endpoint. The last argument is a keyword list with signature options, with the :key and :key_id options being mandatory. For other options, please see with_signature/4. Requires Phoenix.ConnTest."},{"ref":"PlugSignature.ConnTest.html#signed_options/4","title":"PlugSignature.ConnTest.signed_options/4","type":"macro","doc":"Makes a signed request to a Phoenix endpoint. The last argument is a keyword list with signature options, with the :key and :key_id options being mandatory. For other options, please see with_signature/4. Requires Phoenix.ConnTest."},{"ref":"PlugSignature.ConnTest.html#signed_patch/4","title":"PlugSignature.ConnTest.signed_patch/4","type":"macro","doc":"Makes a signed request to a Phoenix endpoint. The last argument is a keyword list with signature options, with the :key and :key_id options being mandatory. For other options, please see with_signature/4. Requires Phoenix.ConnTest."},{"ref":"PlugSignature.ConnTest.html#signed_post/4","title":"PlugSignature.ConnTest.signed_post/4","type":"macro","doc":"Makes a signed request to a Phoenix endpoint. The last argument is a keyword list with signature options, with the :key and :key_id options being mandatory. For other options, please see with_signature/4. Requires Phoenix.ConnTest."},{"ref":"PlugSignature.ConnTest.html#signed_put/4","title":"PlugSignature.ConnTest.signed_put/4","type":"macro","doc":"Makes a signed request to a Phoenix endpoint. The last argument is a keyword list with signature options, with the :key and :key_id options being mandatory. For other options, please see with_signature/4. Requires Phoenix.ConnTest."},{"ref":"PlugSignature.ConnTest.html#signed_trace/4","title":"PlugSignature.ConnTest.signed_trace/4","type":"macro","doc":"Makes a signed request to a Phoenix endpoint. The last argument is a keyword list with signature options, with the :key and :key_id options being mandatory. For other options, please see with_signature/4. Requires Phoenix.ConnTest."},{"ref":"PlugSignature.ConnTest.html#with_digest/2","title":"PlugSignature.ConnTest.with_digest/2","type":"function","doc":"Add an HTTP Digest header (RFC3230, section 4.3.2). When the request body is passed in as a binary, a SHA-256 digest of the body is calculated and added as part of the header. Alternatively, a map of digest types and values may be provided."},{"ref":"PlugSignature.ConnTest.html#with_signature/4","title":"PlugSignature.ConnTest.with_signature/4","type":"function","doc":"Adds an Authorization header with a signature. Requires a secret (RSA private key, EC private key or HMAC shared secret) and key ID. Options :algorithms - the HTTP signature algorithms to be used; list with one or more of: &quot;hs2019&quot; (default) &quot;rsa-sha256&quot; &quot;rsa-sha1&quot; &quot;ecdsa-sha256&quot; &quot;hmac-sha256&quot; The first algorithm in the list will be used to generate the signature (it is a list to allow the core set of configuration options to be shared with PlugSignature in tests). :headers - set the list of HTTP (pseudo) headers to sign; defaults to &quot;(created)&quot; (which is only valid when the algorithm is &quot;hs2019&quot;) :request_target - explicitly set the request target; by default it is built from the Plug.Conn struct (method, request_path and query) :age - shift the HTTP Date header and the signature&#39;s &#39;created&#39; parameter by the given number of seconds into the past; defaults to 0 :created - set the signature&#39;s &#39;created&#39; parameter (overrides :age); set to a empty string to omit the &#39;created&#39; parameter :date - set the HTTP Date header (overrides :age) :expires_in - if set, adds an &#39;expires&#39; parameter with a timestamp the given number of seconds in the future :expires - set the signature&#39;s &#39;expires&#39; parameter (overrides :expires_in) :key_id_override - override the value for keyId in the Authorization header :algorithm_override - override the value for the signature&#39;s &#39;algorithm&#39; parameter in the Authorization header :signature_override - override the signature value sent in the Authorization header :headers_override - override the value for the signature&#39;s &#39;headers&#39; parameter in the Authorization header :created_override - override the value for the signature&#39;s &#39;created&#39; parameter in the Authorization header :expires_override - override the value for the signature&#39;s &#39;expires&#39; parameter in the Authorization header"},{"ref":"PlugSignature.Crypto.html","title":"PlugSignature.Crypto","type":"module","doc":"This module exposes the cryptographic core functions used in HTTP signatures. These functions may be used to implement clients, or alternative server-side implementations, e.g. for Raxx. Supported algorithms: &#39;hs2019&#39;, using ECDSA, RSASSA-PSS or HMAC (all with SHA-512) &#39;rsa-sha256&#39;, using RSASSA-PKCS1-v1_5 &#39;rsa-sha1&#39;, using RSASSA-PKCS1-v1_5 &#39;ecdsa-sha256&#39; &#39;hmac-sha256&#39;"},{"ref":"PlugSignature.Crypto.html#sign/3","title":"PlugSignature.Crypto.sign/3","type":"function","doc":"Generates a signature. Returns {:ok, signature} or {:error, reason}."},{"ref":"PlugSignature.Crypto.html#sign!/3","title":"PlugSignature.Crypto.sign!/3","type":"function","doc":"Generates a signature. Raises in case of an error."},{"ref":"PlugSignature.Crypto.html#verify/4","title":"PlugSignature.Crypto.verify/4","type":"function","doc":""},{"ref":"PlugSignature.Crypto.html#verify!/4","title":"PlugSignature.Crypto.verify!/4","type":"function","doc":"Verifies a signature value. Raises in case of errors."},{"ref":"readme.html","title":"PlugSignature","type":"extras","doc":"PlugSignature Plug for verifying request signatures according to the IETF HTTP signatures draft specification. Supports the following algorithms: &quot;hs2019&quot;, using ECDSA, RSASSA-PSS or HMAC &quot;rsa-sha256&quot;, using RSASSA-PKCS1-v1_5 &quot;ecdsa-sha256&quot; &quot;hmac-sha256&quot; &quot;rsa-sha1&quot;, using RSASSA-PKCS1-v1_5 Development and public release of this package were made possible by Bluecode. The HTTP Date header parsing module was vendored from cowlib, due to build issues that prevented use of the package as a dependency. Cowlib is copyright (c) 2013-2018, Loïc Hoguin"},{"ref":"readme.html#usage","title":"PlugSignature - Usage","type":"extras","doc":"Use PlugSignature in a Phoenix (or other Plug-based) application. Requests with a valid signature are allowed to proceed while all others are rejected. Both the success and the failure behaviour can be customized. PlugSignature requires a callback module that implements the PlugSignature.Callback behaviour. In a Phoenix application this would typically live in a &#39;context&#39; module, and it might look something like this: defmodule MyApp.Auth do import Ecto.Query, only: [from: 2] alias MyApp.Repo alias MyApp.Auth.AccessKey @behaviour PlugSignature.Callback @impl true def client_lookup(key_id, &quot;hs2019&quot;, _conn) do query = from a in AccessKey, where: a.key_id == ^key_id, preload: :client case Repo.one(query) do nil -&gt; {:error, &quot;Invalid access key ID: \#{key_id}&quot;} {:ok, %AccessKey{revoked: true}} -&gt; {:error, &quot;Access key revoked: \#{key_id}&quot;} {:ok, %AccessKey{public_key: pem, client: client}} -&gt; public_key = X509.PublicKey.from_pem!(pem) {:ok, client, public_key} end end end To enable verification of the request body, through the HTTP Digest header, add PlugBodyDigest from the plug_body_digest package, e.g. to the application&#39;s Phoenix Endpoint: defmodule MyAppWeb.Endpoint do # ... plug Plug.Parsers, parsers: [:urlencoded, :multipart, :json], pass: [&quot;*/*&quot;], json_decoder: Phoenix.json_library(), body_reader: {PlugBodyDigest, :digest_body_reader, []} plug PlugBodyDigest end Finally, add PlugSignature, for instance to a Phoenix Router pipeline: defmodule MyAppWeb.Router do # ... pipeline :api do plug :accepts, [&quot;json&quot;] plug PlugSignature, callback_module: MyApp.Auth, headers: &quot;(request-target) (created) host digest&quot;, on_success: {PlugSignature, :assign_client, [:client]} end # ... end Alternatively it may be used inside a controller&#39;s pipeline, possibly with guards: defmodule MyAppWeb.SomeController do use MyAppWeb, :controller plug PlugSignature, [ callback_module: MyApp.Auth, headers: &quot;(request-target) (created) host digest&quot; ] when not action in [:show, :index] # ... end The directory plug_signature_example in the package source repository contains a minimal functional sample application, implemented as a simple Plug server that echos back the request parameters after signature authentication."},{"ref":"readme.html#client-implementation","title":"PlugSignature - Client implementation","type":"extras","doc":"The sample application includes clients written in Elixir, using Tesla middleware, and as a shell script, using OpenSSL and cURL."},{"ref":"readme.html#installation","title":"PlugSignature - Installation","type":"extras","doc":"Add plug_signature to your list of dependencies in mix.exs (and consider adding plug_body_digest as well): def deps do [ {:plug_body_digest, &quot;~&gt; 0.5.0&quot;}, {:plug_signature, &quot;~&gt; 0.5.0&quot;} ] end Documentation can be found at https://hexdocs.pm/plug_signature."},{"ref":"readme.html#license","title":"PlugSignature - License","type":"extras","doc":"Copyright (c) 2019, Bram Verburg All rights reserved. Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS &quot;AS IS&quot; AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."}]